// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItem.hpp"
#include "component/display/IRenderableComponent.hpp"
#include "display/shader/UIShader.hpp"

namespace Dawn {
  class UIComponent;

  class UICanvas :
    public SceneComponent,
    public IRenderableComponent
  {
    private:
      std::shared_ptr<Mesh> mesh;
      UIShaderData data;

      size_t quadCount = 0;
      shadertexturebinding_t nextBinding = 0;
      std::unordered_map<
        shadertexturebinding_t, std::shared_ptr<Texture>
      > textures;
      std::map<
        std::shared_ptr<Texture>, shadertexturebinding_t
      > textureBindings;
      
      std::vector<std::shared_ptr<IRenderPass>> passes;
      
    protected:
      virtual void onInit() override;
      virtual void onDispose() override;
      void flushPass();

    public:
      std::vector<std::shared_ptr<UIComponent>> components;

      std::vector<std::shared_ptr<IRenderPass>> getPasses(
        struct RenderPassContext &ctx
      ) override;

      void addQuad(
        const glm::vec4 quad,
        const glm::vec4 uvs,
        const struct Color color,
        const std::shared_ptr<Texture> texture
      );
  };
}